// Wrapper to emulate Chrome APIs in Firefox (Manifest V2 context)
(function() {
    // Ensure `chrome` namespace exists for compatibility
    if (typeof chrome === 'undefined') {
        var chrome = {};
    }

    // Tabs API
    chrome.tabs = {
        update: function(tabId, updateProperties, callback) {
            browser.tabs.update(tabId, updateProperties).then(() => {
                if (callback) callback();
            }).catch(console.error);
        },
        create: function(createProperties, callback) {
            browser.tabs.create(createProperties).then(tab => {
                if (callback) callback(tab);
            }).catch(console.error);
        }
    };

    // WebRequest API
    chrome.webRequest = {
        onBeforeRequest: {
            addListener: function(listener, filter, extraInfoSpec) {
                browser.webRequest.onBeforeRequest.addListener(listener, filter, extraInfoSpec);
            }
        }
    };

    // Runtime API
    chrome.runtime = {
        onStartup: {
            addListener: function(listener) {
                browser.runtime.onStartup.addListener(listener);
            }
        },
        onInstalled: {
            addListener: function(listener) {
                browser.runtime.onInstalled.addListener(listener);
            }
        },
        openOptionsPage: function() {
            browser.runtime.openOptionsPage();
        },
        lastError: null // Placeholder for compatibility
    };

    // Alarms API
    chrome.alarms = {
        create: function(name, alarmInfo) {
            browser.alarms.create(name, alarmInfo);
        },
        onAlarm: {
            addListener: function(listener) {
                browser.alarms.onAlarm.addListener(listener);
            }
        }
    };

    // Storage API
    chrome.storage = {
        sync: {
            get: function(keys, callback) {
                browser.storage.sync.get(keys).then(callback).catch(err => {
                    chrome.runtime.lastError = err;
                    console.error(err);
                });
            },
            set: function(items, callback) {
                browser.storage.sync.set(items).then(() => {
                    if (callback) callback();
                }).catch(err => {
                    chrome.runtime.lastError = err;
                    console.error(err);
                });
            }
        },
        onChanged: {
            addListener: function(listener) {
                browser.storage.onChanged.addListener(listener);
            }
        }
    };

    // Downloads API
    /*chrome.downloads = {
        cancel: function(downloadId, callback) {
            browser.downloads.cancel(downloadId).then(() => {
                if (callback) callback();
            }).catch(err => {
                chrome.runtime.lastError = err;
                console.error(err);
            });
        },
        onDeterminingFilename: {
            addListener: function(listener) {
                browser.downloads.onDeterminingFilename.addListener(listener);
            }
        }
    };*/

    // Context Menus API
    chrome.contextMenus = {
        create: function(createProperties, callback) {
            return browser.contextMenus.create(createProperties, callback);
        },
        onClicked: {
            addListener: function(listener) {
                browser.contextMenus.onClicked.addListener(listener);
            }
        }
    };

    // DeclarativeNetRequest API
    chrome.declarativeNetRequest = {
        updateDynamicRules: function(ruleOptions, callback) {
            browser.declarativeNetRequest.updateDynamicRules(ruleOptions).then(() => {
                if (callback) callback();
            }).catch(err => {
                chrome.runtime.lastError = err;
                console.error(err);
            });
        },
        getDynamicRules: function(callback) {
            browser.declarativeNetRequest.getDynamicRules().then(rules => {
                callback(rules);
            }).catch(err => {
                chrome.runtime.lastError = err;
                console.error(err);
            });
        }
    };

    // Action API
    chrome.action = {
        onClicked: {
            addListener: function(listener) {
                browser.action.onClicked.addListener(listener);
            }
        }
    };
    window.chrome = chrome;
})();
